"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TypeError_1 = require("./TypeError");
/**
 * Resolve a raw value into a string.
 *
 * @param value Raw value.
 * @returns String or undefined if value is undefined.
 */
function resolveString(value) {
    if (typeof value === 'string') {
        return value;
    }
    if (typeof value !== 'undefined') {
        return JSON.stringify(value);
    }
}
exports.resolveString = resolveString;
/**
 * Resolve a raw value into a number.
 *
 * @param value Raw value.
 * @returns Number or undefined if value is undefined.
 * @throws TypeError if the value is defined but cannot be parsed as a number.
 */
function resolveNumber(value) {
    if (typeof value === 'number' && !isNaN(value)) {
        return value;
    }
    if (typeof value === 'string') {
        const parsedValue = parseFloat(value);
        if (!isNaN(parsedValue)) {
            return parsedValue;
        }
    }
    if (typeof value !== 'undefined') {
        throw new TypeError_1.default(`not a number: ${value}`);
    }
}
exports.resolveNumber = resolveNumber;
/**
 * Resolve a raw value into an object.
 *
 * @param value Raw value.
 * @returns Object or undefined if value is undefined.
 * @throws TypeError if the value is defined but cannot be parsed as an object.
 */
function resolveObject(value) {
    if (typeof value === 'object') {
        return value;
    }
    if (typeof value === 'string' && value) {
        try {
            return JSON.parse(value);
        }
        catch (e) {
            throw new TypeError_1.default(`invalid JSON: ${value}`, e);
        }
    }
    if (typeof value !== 'undefined') {
        throw new TypeError_1.default(`not an object: ${value}`);
    }
}
exports.resolveObject = resolveObject;
/**
 * Create a resolver for an instantiable type.
 *
 * @param newable Constructor of the type to instantiate.
 * @returns Resolver for type Newable<T>.
 */
function resolveType(newable) {
    /**
     * Resolve a raw value into an instantiable type.
     *
     * @param value Raw value.
     * @returns Instance of Newable<T> or undefined if value is undefined.
     * @throws TypeError if the constructor of newable throws an error.
     */
    return (value) => {
        if (typeof value === 'undefined') {
            return;
        }
        try {
            return new newable(value);
        }
        catch (e) {
            throw new TypeError_1.default(`failed to create ${newable.name}`, e);
        }
    };
}
exports.resolveType = resolveType;
//# sourceMappingURL=resolvers.js.map