"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const merge = require("lodash.merge");
const TypeError_1 = require("./TypeError");
function isSupplier(value) {
    return (typeof value === 'object' &&
        'get' in value &&
        typeof value.get === 'function' &&
        'aggregate' in value &&
        typeof value.aggregate === 'function');
}
exports.isSupplier = isSupplier;
class ObjectSupplier {
    constructor(storage) {
        this.storage = Object.assign({}, storage);
    }
    get(name) {
        return this.storage[name];
    }
    aggregate() {
        return Object.assign({}, this.storage);
    }
}
exports.ObjectSupplier = ObjectSupplier;
function getValueOrNext(name, supplier, next, resolve = (_) => _) {
    const value = resolve(supplier.get(name));
    if (value === undefined) {
        return next(name, resolve);
    }
    // Merge multiple object-values with the same name in hierarchically nested stores.
    if (typeof value === 'object' && !Array.isArray(value)) {
        const nextValue = next(name, resolve);
        if (typeof nextValue === 'object' && !Array.isArray(nextValue)) {
            return merge({}, nextValue, value);
        }
    }
    return value;
}
exports.getValueOrNext = getValueOrNext;
function peekValueOrNext(name, supplier, next) {
    return typeof supplier.get(name) !== 'undefined' || next(name, true);
}
exports.peekValueOrNext = peekValueOrNext;
function createStore(supplier, nameModifier = _ => _) {
    return {
        get(name, next, resolve) {
            return getValueOrNext(nameModifier(name), supplier, next, resolve);
        },
        has(name, next) {
            return peekValueOrNext(nameModifier(name), supplier, next);
        },
        aggregate() {
            return supplier.aggregate();
        }
    };
}
exports.createStore = createStore;
function createAccessor(store, next) {
    return (name, arg1 = (_) => _) => {
        if (arg1 === true) {
            return store.has(name, next);
        }
        else {
            return store.get(name, next, arg1);
        }
    };
}
exports.createAccessor = createAccessor;
exports.baseAccessor = (_, arg1) => {
    if (arg1 === true)
        return false;
    else if (arg1)
        return arg1(undefined);
    else
        return undefined;
};
function aggregateContainerValues(container, prefix, separator, keyTransform) {
    const aggregate = {};
    for (const key of Object.keys(container).filter(k => k.startsWith(prefix) && typeof container[k] !== 'undefined')) {
        const subKeys = key
            .replace(prefix, '')
            .split(separator)
            .filter(k => k);
        if (subKeys.length > 1) {
            let currentAggregate = aggregate;
            for (let i = 0; i < subKeys.length; i += 1) {
                const k = keyTransform(subKeys[i]);
                if (i < subKeys.length - 1) {
                    currentAggregate[k] = aggregate[k] || {};
                    currentAggregate = currentAggregate[k];
                }
                else {
                    currentAggregate[k] = container[key];
                }
            }
        }
        else {
            const k = keyTransform(key.replace(prefix, ''));
            aggregate[k] = container[key];
        }
    }
    return aggregate;
}
exports.aggregateContainerValues = aggregateContainerValues;
function randomString() {
    return Math.random()
        .toString(36)
        .substring(2, 10);
}
exports.randomString = randomString;
function assertString(value) {
    if (typeof value === 'string' || typeof value === 'undefined') {
        return value;
    }
    throw new TypeError_1.default(`expected value to be string but was ${typeof value}`);
}
exports.assertString = assertString;
function assertNumber(value) {
    if ((typeof value === 'number' && !isNaN(value)) || typeof value === 'undefined') {
        return value;
    }
    throw new TypeError_1.default(`expected value to be number but was ${typeof value}`);
}
exports.assertNumber = assertNumber;
function assertObject(value) {
    if (typeof value === 'object' || typeof value === 'undefined') {
        return value;
    }
    throw new TypeError_1.default(`expected value to be object but was ${typeof value}`);
}
exports.assertObject = assertObject;
function assertType(value, newable) {
    if (value instanceof newable || typeof value === 'undefined') {
        return value;
    }
    throw new TypeError_1.default(`expected value to be ${newable.name} but was ${typeof value}`);
}
exports.assertType = assertType;
//# sourceMappingURL=util.js.map